"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.diveThroughMockHocs = exports.diveSingleToComponent = exports.diveStaticAmount = exports.diveThroughComponent = exports.diveThroughComponents = exports.MAX_DIVES = void 0;

var _enzyme = require("enzyme");

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var MAX_DIVES = 50;
exports.MAX_DIVES = MAX_DIVES;

var invalidMessage = function invalidMessage(methodName, component, otherName, otherValue) {
  throw "".concat(methodName, ": either the component (").concat(component, ") or the ").concat(otherName, " (").concat(otherValue, ") is invalid");
};

var ProviderDiver_legacy = function ProviderDiver_legacy(diveTo) {
  return function (wrappedComponent) {
    return (0, _enzyme.shallow)(wrappedComponent).find(diveTo).dive();
  };
};

var ProviderDeepDiver_legacy = function ProviderDeepDiver_legacy(diveTo) {
  return function (wrappedComponent) {
    var component = (0, _enzyme.shallow)(wrappedComponent);

    _lodash["default"].forEach(diveTo, function (next) {
      component = component.find(next).dive();
    });

    return component;
  };
};

var diveThroughComponents = function diveThroughComponents(shallowComponent, diveTos) {
  if (!shallowComponent || !diveTos) throw invalidMessage("diveThroughComponents", shallowComponent, "diveTos", diveTos);
  var component = shallowComponent;

  _lodash["default"].forEach(diveTo, function (next) {
    component = component.find(next).dive();
  });

  return component;
};

exports.diveThroughComponents = diveThroughComponents;

var diveThroughComponent = function diveThroughComponent(shallowComponent, diveTo) {
  if (!shallowComponent || !diveTo) throw invalidMessage("diveThroughComponent", shallowComponent, "diveTo", diveTo);
  return shallowComponent.find(diveTo).dive();
};

exports.diveThroughComponent = diveThroughComponent;

var diveStaticAmount = function diveStaticAmount(shallowComponent, diveCount) {
  if (!shallowComponent || !_lodash["default"].isNumber(diveCount)) invalidMessage("diveStaticAmount", shallowComponent, "diveCount", diveCount);
  var comp = shallowComponent;

  for (var i = 0; i < diveCount; i++) {
    comp = comp.dive();
  }

  return comp;
};

exports.diveStaticAmount = diveStaticAmount;

var diveSingleToComponent = function diveSingleToComponent(shallowComponent, diveTarget) {
  var comp = shallowComponent;
  if (!diveTarget || !shallowComponent) invalidMessage("diveSingleToComponent", shallowComponent, "diveTarget", diveTarget);

  for (var i = 0; i < MAX_DIVES && comp.name() != diveTarget; i++) {
    comp = comp.dive();
  }

  return comp.dive();
};

exports.diveSingleToComponent = diveSingleToComponent;

var diveThroughMockHocs = function diveThroughMockHocs(shallowComponent) {
  if (!shallowComponent) throw "diveThroughMocHoc: the component was invalid (".concat(shallowComponent, ")");
  var comp = shallowComponent;

  for (var i = 0; i < MAX_DIVES && comp.name() == ""; i++) {
    comp = comp.dive();
  }

  return comp.dive();
};

exports.diveThroughMockHocs = diveThroughMockHocs;