"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.jestMockHocMethod = void 0;

var _react = _interopRequireDefault(require("react"));

var _lodash = _interopRequireDefault(require("lodash"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var path = require("path");

var stack = require("callsite"); // * -------------- Higher-Order-Component Mocking -------------- *


var jestMockHocMethod = function jestMockHocMethod(path, method) {
  return jest.doMock(path, function () {
    return method;
  });
};

exports.jestMockHocMethod = jestMockHocMethod;

var MockHOC = function MockHOC(path, methodName, mockFunction) {
  return function (hocProp) {
    if (!path || typeof path !== "string") throw "MockHoc | MockHocHierarchy: the path provided was not valid, but instead: ".concat(path); // ? if no method name is specified, then the mock method will be directly assigned to the mockMethod.
    // ?? otherwise, a sub-object named the >methodName< will be injected into the mockMethod, containing the mock method

    var wrapMethod = function wrapMethod(WrappedComponent) {
      return function (props) {
        if (hocProp) props = _lodash["default"].assign(hocProp, props);
        return _react["default"].createElement(WrappedComponent, props);
      };
    };

    var mockMethod = {};
    if (methodName) mockMethod[methodName] = wrapMethod;else mockMethod = wrapMethod; // mockMethod[methodName] = WrappedComponent => props => {
    //   if (hocProp) props = _.assign(hocProp, props);
    //   return <WrappedComponent {...props} />;
    // };

    mockFunction(path, mockMethod); //jest.doMock(path, () => mockMethod);
  };
};

var MockHOCHierarchy = function MockHOCHierarchy(mocks) {
  return function (requireName) {
    if (!Array.isArray(mocks)) throw "MockHocHierarchy: the mocked data is not an arrays, but instead: ".concat(mocks);
    mocks.forEach(function (mock) {
      var path = mock.path,
          methodName = mock.methodName,
          hocProp = mock.hocProp,
          contextModuleMethod = mock.contextModuleMethod;
      MockHOC(path, methodName, contextModuleMethod)(hocProp);
    });
    return require(requireName);
  };
};

var MockHoc =
/*#__PURE__*/
function () {
  function MockHoc(wrappedComponentPath, origin, clearOnCreation) {
    _classCallCheck(this, MockHoc);

    this.mock = this.mock.bind(this);
    this["with"] = this["with"].bind(this);
    this.clear = this.clear.bind(this);
    this.create = this.create.bind(this);
    this.createPure = this.createPure.bind(this);
    if (clearOnCreation) this.clear();
    this.requiredPath = wrappedComponentPath;
    this.origin = origin;
    this.imported = []; // ? this parses the file name to potential get the function/class name of the "imported" module
    // ?? this is used, in case the provided apply method does not find an export with the name of >target<

    this.potentialRequireMethodName = (0, _utils.GetFileName)(wrappedComponentPath); // ? if the origin is not specifided, then try to determine it through the call-stack

    if (!this.origin) {
      var sites = stack();
      if (sites.length >= 3) this.origin = path.dirname(sites[2].getFileName());
    } // ? if the origin does not end with a trailing '/', then one needs to be added to reduce failure from origin + path


    if (this.origin && this.origin[this.origin.length - 1] != "/") this.origin += "/";
  }

  _createClass(MockHoc, [{
    key: "mock",
    value: function mock(path, methodName) {
      var contextModuleMethod = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      if (!path || typeof path !== "string") throw "MockHoc: [mock] missing path, instead recieved ".concat(path); // ? if no method is specified then try and parse it out of the path

      if (methodName == "#uc" || methodName == "#lc") methodName = (0, _utils.SetFirstCase)((0, _utils.GetFileName)(path), methodName);

      if (typeof contextModuleMethod !== "function") {
        path = this.origin + path;
        contextModuleMethod = jestMockHocMethod;
      } // ? store it for when applied is called


      this.imported.push({
        path: path,
        methodName: methodName,
        hocProp: {},
        contextModuleMethod: contextModuleMethod
      });
      return this;
    }
  }, {
    key: "with",
    value: function _with(injectedProps) {
      var importCount = this.imported.length;
      if (importCount < 1) return;
      this.imported[importCount - 1].hocProp = _lodash["default"].assign(this.imported[importCount - 1].hocProp, injectedProps);
      return this;
    }
  }, {
    key: "clear",
    value: function clear() {
      jest.resetModules();
      return this;
    }
  }, {
    key: "createPure",
    value: function createPure() {
      return MockHOCHierarchy(this.imported)(this.origin + this.requiredPath);
    }
  }, {
    key: "create",
    value: function create() {
      var target = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "default";
      var req = MockHOCHierarchy(this.imported)(this.origin + this.requiredPath); // ? if the specified target is not found, then attempt to find a predicted target name instead
      // ?? if it finds neither, then throw a detailed, helpful error (as opposed to how jest handles it)

      if (!req[target]) {
        if (!req[this.potentialRequireMethodName]) throw "MockHoc [create]: could not find the exported target, \"".concat(target, "\", nor could it find the fallback target, \"").concat(this.potentialRequireMethodName, "\"");
        target = this.potentialRequireMethodName;
      }

      return req[target];
    }
  }]);

  return MockHoc;
}();

var constructMockHoc = function constructMockHoc(wrappedComponentPath) {
  var origin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var clearOnCreation = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  return new MockHoc(wrappedComponentPath, origin, clearOnCreation);
};

var _default = constructMockHoc;
exports["default"] = _default;